Package: lumberjack
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: EUPL
Title: Track Changes in Data
Type: Package
LazyLoad: yes
Authors@R: c(person("Mark", "van der Loo"
                  , role    = c("aut","cre")
                  , email   = "mark.vanderloo@gmail.com"
                  , comment = c(ORCID="0000-0002-9807-4686"))
           , person("Floris", "Ruijter", role = "ctb") )
Description: A framework that allows for easy logging of changes in data.
    Main features: start tracking changes by adding a single line of code to 
    an existing script. Track changes in multiple datasets, using multiple 
    loggers. Add custom-built loggers or use loggers offered by other 
    packages. <doi:10.18637/jss.v098.i01>.
Version: 1.3.1
URL: https://github.com/markvanderloo/lumberjack
BugReports: https://github.com/markvanderloo/lumberjack/issues
Imports: utils, R6
Depends: R (>= 3.4.0)
Suggests: tinytest
RoxygenNote: 7.2.1
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-03-29 08:35:26 UTC; mark
Author: Mark van der Loo [aut, cre] (<https://orcid.org/0000-0002-9807-4686>),
  Floris Ruijter [ctb]
Repository: CRAN
Date/Publication: 2023-03-29 08:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 18:08:09 UTC; unix
