# Generated by roxygen2: do not edit by hand

export("%>>%")
export("%L>%")
export(cellwise)
export(dump_log)
export(expression_logger)
export(filedump)
export(get_log)
export(no_log)
export(run_file)
export(simple)
export(source_file)
export(start_log)
export(stop_log)
importFrom(R6,R6Class)
importFrom(utils,capture.output)
